package com.ruoyi.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.toolkit.Db;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.domain.ReviewSceneChangeTask;
import com.ruoyi.domain.ReviewSceneChangeTaskUserRelation;
import com.ruoyi.framework.web.domain.server.Sys;
import com.ruoyi.mapper.ReviewKeyPointMapper;
import com.ruoyi.service.ReviewSceneChangeTaskService;
import com.ruoyi.mapper.ReviewSceneChangeTaskMapper;
import com.ruoyi.service.ReviewSceneChangeTaskUserRelationService;
import com.ruoyi.service.ReviewSceneService;
import com.ruoyi.service.ReviewTaskInboxService;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.web.request.ReviewSceneChangeTaskFindCreateRequest;
import com.ruoyi.web.request.ReviewSceneChangeTaskFindFinishRequest;
import com.ruoyi.web.request.ReviewSceneChangeTaskFindPendingRequest;
import com.ruoyi.web.request.ReviewSceneChangeTaskPassRequest;
import com.ruoyi.web.response.ReviewDetailsListResponse;
import com.ruoyi.web.response.ReviewKeyPointListResponse;
import com.ruoyi.web.response.ReviewSceneChangeTaskResponse;
import com.ruoyi.web.response.ReviewerListResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
* @author wangfei
* @description 针对表【t_review_scene_change_task(审查场景变更任务)】的数据库操作Service实现
* @createDate 2023-12-14 14:28:17
*/
@Transactional
@Service
public class ReviewSceneChangeTaskServiceImpl extends ServiceImpl<ReviewSceneChangeTaskMapper, ReviewSceneChangeTask> implements ReviewSceneChangeTaskService {

    @Autowired
    private ReviewSceneChangeTaskUserRelationService reviewSceneChangeTaskUserRelationService;

    @Autowired
    private ReviewSceneChangeTaskService reviewSceneChangeTaskService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ReviewSceneService reviewSceneService;

    @Autowired
    private ReviewTaskInboxService reviewTaskInboxService;

    @Autowired
    private ReviewSceneChangeTaskMapper reviewSceneChangeTaskMapper;

    @Autowired
    private SysUserRoleMapper sysUserRoleMapper;

    @Autowired
    private ReviewKeyPointMapper reviewKeyPointMapper;
    @Override
    public void create(ReviewSceneChangeTask reviewSceneChangeTask) {

        // 获取当前登陆人
        SysUser loginUser = SecurityUtils.getLoginUser().getUser();

        // 创建任务
        reviewSceneChangeTask
                .setTaskInitiator(loginUser.getNickName())
                .setTaskInitiatorDept(loginUser.getDept().getDeptName())
                .setTaskInitiatorId(loginUser.getUserId())
                .setTaskStatus(ReviewSceneChangeTask.STATUS_PENDING)
                .setTaskBeginTime(new Date());

        //获取operationSort设置operation
        String[] operationSortPart = reviewSceneChangeTask.getOperationSort().split("_");
        reviewSceneChangeTask.setOperation(operationSortPart[0]);

        // 保存任务
        save(reviewSceneChangeTask);

        /*// 绑定创新基地人员
        List<SysUser> InnovationSysUsers = sysUserService.selectUserListByRoleId(ReviewSceneChangeTask.INNOVATION_BASE_ID);
        List<ReviewSceneChangeTaskUserRelation> InnovationRelations = new ArrayList();
        for(SysUser sysUser : InnovationSysUsers) {
            ReviewSceneChangeTaskUserRelation relation = new ReviewSceneChangeTaskUserRelation()
                    .setTaskId(reviewSceneChangeTask.getId())
                    .setUserId(sysUser.getUserId())
                    .setName(sysUser.getNickName())
                    .setDept(sysUser.getDept().getDeptName())
                    .setRoleId(ReviewSceneChangeTask.INNOVATION_BASE_ID);
            InnovationRelations.add(relation);
        }

        reviewSceneChangeTaskUserRelationService.saveBatch(InnovationRelations);*/

        // 绑定审核人
        List<SysUser> sysUsers = sysUserService.selectUserListByRoleId(ReviewSceneChangeTask.REVIEW_ROLE_ID);
        List<ReviewSceneChangeTaskUserRelation> relations = new ArrayList();
        for(SysUser sysUser : sysUsers) {
            ReviewSceneChangeTaskUserRelation relation = new ReviewSceneChangeTaskUserRelation()
                    .setTaskId(reviewSceneChangeTask.getId())
                    .setUserId(sysUser.getUserId())
                    .setName(sysUser.getNickName())
                    .setDept(sysUser.getDept().getDeptName())
                    .setRoleId(ReviewSceneChangeTask.REVIEW_ROLE_ID);
            relations.add(relation);
        }

        reviewSceneChangeTaskUserRelationService.saveBatch(relations);
    }

    @Override
    public void pass(Long taskId, String comment) {
        doProcess(taskId, comment, ReviewSceneChangeTask.RESULT_PASS);
    }



    @Override
    public void reject(Long taskId, String comment) {
        doProcess(taskId, comment, ReviewSceneChangeTask.RESULT_REJECT);
    }

    @Override
    public void audit(ReviewSceneChangeTaskPassRequest request) {
        if (request.getPassed().equals(ReviewSceneChangeTask.PASSED_REJECT)) {
            doProcess(request.getTaskId(), request.getComment(), ReviewSceneChangeTask.RESULT_REJECT);
        } else if(request.getPassed().equals(ReviewSceneChangeTask.PASSED_PASS)){
            doProcess(request.getTaskId(), request.getComment(), ReviewSceneChangeTask.RESULT_PASS);
        } else {
            throw new ServiceException("审核状态不正确",HttpStatus.ERROR);
        }
    }

    public void doProcess(Long taskId, String comment, String decision) {

        ReviewSceneChangeTask task = getById(taskId);
        checkTaskStatus(task.getTaskStatus());

        // 获取当前登陆人
        SysUser loginUser = SecurityUtils.getLoginUser().getUser();

        // 1.获取该任务的所有审批人
        // 2.获取当前用户对应的审批信息
        // 3.保存评审意见
//        List<ReviewSceneChangeTaskUserRelation> relations = reviewSceneChangeTaskUserRelationService.findListByTaskId(taskId);
        List<ReviewSceneChangeTaskUserRelation> relations = Db.lambdaQuery(ReviewSceneChangeTaskUserRelation.class)
                .eq(ReviewSceneChangeTaskUserRelation::getTaskId, taskId)
                .eq(ReviewSceneChangeTaskUserRelation::getRoleId, ReviewSceneChangeTask.REVIEW_ROLE_ID)
                .list();
        for(ReviewSceneChangeTaskUserRelation relation : relations) {
            if(relation.getUserId().equals(loginUser.getUserId())) {
                relation.setDecision(decision);
                relation.setComment(comment);
                relation.setDecisionTime(new Date());
                reviewSceneChangeTaskUserRelationService.updateById(relation);
                break;
            }
        }

        // 判断当前任务是否全员审核完毕，如果是则将当前任务设置为完成。
        if(isFinish(relations)) {

            task.setTaskStatus(ReviewSceneChangeTask.STATUS_FINISH);
            task.setTaskEndTime(new Date());
            task.setTaskResult(isPassed(relations) ? ReviewSceneChangeTaskUserRelation.DECISION_PASS : ReviewSceneChangeTaskUserRelation.DECISION_REJECT);

            reviewSceneChangeTaskService.updateById(task);

            // 如果当前任务为全员通过，则对审查标准文件进行更新，同时向受到变更影响的问卷的“收件箱”表中写入数据，记录变更用于提醒用户。
            if(task.getTaskResult().equals(ReviewSceneChangeTaskUserRelation.DECISION_PASS)) {
                // 更新审核标准
                reviewSceneService.change(task);

                //场景变更时发送信息
                if (Objects.equals(task.getOperationSort(), "ADD_SCENE") || Objects.equals(task.getOperationSort(), "DELETE_SCENE")) {
                    // 通知受影响的问卷
                    reviewTaskInboxService.saveNotify(taskId, task.getType());
                }

            }
        }

    }

    @Override
    public void close(Long taskId) {
    }

    @Override
    public List<ReviewSceneChangeTask> findLastChangeByTaskId(Long taskId) {
        return reviewSceneChangeTaskMapper.findLastChangeByTaskId(taskId);
    }

    private void checkTaskStatus(String status) {
        if(status != null && ReviewSceneChangeTask.STATUS_FINISH.equals(status)) {
            throw new ServiceException("不能处理一个已经完成的任务", HttpStatus.ERROR);
        }
        if(status != null && ReviewSceneChangeTask.STATUS_CLOSE.equals(status)) {
            throw new ServiceException("不能处理一个已经关闭的任务", HttpStatus.ERROR);
        }
    }

    /**
     * 检查当前任务是否完成（所有审核人都发表了意见）
     * @param relations
     * @return
     */
    private boolean isFinish(List<ReviewSceneChangeTaskUserRelation> relations) {

        for(ReviewSceneChangeTaskUserRelation relation : relations) {
            String decision = relation.getDecision();
            if(decision == null || "".equals(decision)) {
                return false;
            }
        }

        return true;
    }

    /**
     * 检查是否全员通过
     * @param relations
     * @return
     */
    private boolean isPassed(List<ReviewSceneChangeTaskUserRelation> relations) {

        for(ReviewSceneChangeTaskUserRelation relation : relations) {
            if(relation.getDecision().equals(ReviewSceneChangeTaskUserRelation.DECISION_REJECT)) {
                return false;
            }
        }

        return true;
    }

    /**
     * 根据用户ID查询待办列表
     * @param userId
     * @return
     */
    @Override
    public List<ReviewSceneChangeTaskResponse> findPendingListByUserId(Long userId, ReviewSceneChangeTaskFindPendingRequest request) {
        List<ReviewSceneChangeTaskResponse> reviewSceneChangeTask = reviewSceneChangeTaskMapper.findPendingListByUserId(userId,request);

        //查询审核人员
        for (ReviewSceneChangeTaskResponse task : reviewSceneChangeTask){
            Long id = Long.parseLong(task.getId());
            List<ReviewerListResponse> reviewerListResponseList = reviewSceneChangeTaskMapper.selectReviewerListByTaskId(id);
            Set<String> reviewerNameSet = new HashSet<>();
            for (ReviewerListResponse reviewer : reviewerListResponseList){
                reviewerNameSet.add(reviewer.getReviewer());
            }
            //生成审核人员名字字符串
            String reviewerNames = String.join("、", reviewerNameSet);
            task.setReviewer(reviewerNames);
        }
        /*List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.REVIEW_ROLE_ID);
        List<SysUser> reviewerList = sysUserService.selectUserListByIds(reviewerIdList);
        Set<String> reviewerNameSet = new HashSet<>();
        for (SysUser reviewer : reviewerList){
            reviewerNameSet.add(reviewer.getNickName());
        }

        String reviewerNames = String.join("、", reviewerNameSet);

        for (ReviewSceneChangeTaskResponse taskResponse : reviewSceneChangeTask){
            taskResponse.setReviewer(reviewerNames);
        }*/
        return reviewSceneChangeTask;
    }

    /**
     * 根据用户ID查询已办列表
     * @param userId
     * @return
     */
    @Override
    public List<ReviewSceneChangeTaskResponse> findFinishListByUserId(Long userId, ReviewSceneChangeTaskFindFinishRequest request) {
        List<ReviewSceneChangeTaskResponse> reviewSceneChangeTask = reviewSceneChangeTaskMapper.findFinishListByUserId(userId,request);

        //查询审核人员
        for (ReviewSceneChangeTaskResponse task : reviewSceneChangeTask){
            Long id = Long.parseLong(task.getId());
            List<ReviewerListResponse> reviewerListResponseList = reviewSceneChangeTaskMapper.selectReviewerListByTaskId(id);
            Set<String> reviewerNameSet = new HashSet<>();
            for (ReviewerListResponse reviewer : reviewerListResponseList){
                reviewerNameSet.add(reviewer.getReviewer());
            }
            //生成审核人员名字字符串
            String reviewerNames = String.join("、", reviewerNameSet);
            task.setReviewer(reviewerNames);
        }
        /*List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.REVIEW_ROLE_ID);
        List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.REVIEW_ROLE_ID);
        List<SysUser> reviewerList = sysUserService.selectUserListByIds(reviewerIdList);
        Set<String> reviewerNameSet = new HashSet<>();
        for (SysUser reviewer : reviewerList){
            reviewerNameSet.add(reviewer.getNickName());
        }
        String reviewerNames = String.join("、", reviewerNameSet);
        for (ReviewSceneChangeTaskResponse taskResponse: reviewSceneChangeTask){
            taskResponse.setReviewer(reviewerNames);
        }*/
        return reviewSceneChangeTask;
    }

    /**
     * 查询当前用户发布的任务
     * @param userId
     * @return
     */
    @Override
    public List<ReviewSceneChangeTask> findCreateListByUserId(Long userId, ReviewSceneChangeTaskFindCreateRequest request) {
        List<ReviewSceneChangeTask> reviewSceneChangeTask = reviewSceneChangeTaskMapper.findCreateListByUserId(userId,request);
        //查询审核人员
        /*List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.REVIEW_ROLE_ID);*/
        for (ReviewSceneChangeTask task : reviewSceneChangeTask){
            List<ReviewerListResponse> reviewerListResponseList = reviewSceneChangeTaskMapper.selectReviewerListByTaskId(task.getId());
            Set<String> reviewerNameSet = new HashSet<>();
            for (ReviewerListResponse reviewer : reviewerListResponseList){
                reviewerNameSet.add(reviewer.getReviewer());
            }
            // 生成审核人员名字字符串
            String reviewerNames = String.join("、", reviewerNameSet);
            task.setReviewer(reviewerNames);
        }
        return reviewSceneChangeTask;
    }

    @Override
    public ReviewSceneChangeTask getBySceneChangeTaskId(Long id) {
        ReviewSceneChangeTask reviewSceneChangeTask = reviewSceneChangeTaskMapper.findSceneChangeTaskById(id);
        List<ReviewerListResponse> reviewerListResponseList = reviewSceneChangeTaskMapper.selectReviewerListByTaskId(id);
        Set<String> reviewerNameSet = new HashSet<>();
        for (ReviewerListResponse reviewer : reviewerListResponseList){
            reviewerNameSet.add(reviewer.getReviewer());
        }
        String reviewerNames = String.join("、", reviewerNameSet);
        reviewSceneChangeTask.setReviewer(reviewerNames);
        /*//查询审核人员
        List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.REVIEW_ROLE_ID);
        List<SysUser> reviewerList = sysUserService.selectUserListByIds(reviewerIdList);

        Set<String> reviewerNameSet = new HashSet<>();
        for (SysUser reviewer : reviewerList) {
            reviewerNameSet.add(reviewer.getNickName());
        }

        // 生成审核人员名字字符串
        String reviewerNames = String.join("、", reviewerNameSet);
        reviewSceneChangeTask.setReviewer(reviewerNames);*/
        return reviewSceneChangeTask;
    }

    /**
     * 创新基地查询待办列表
     * @return
     */
    @Override
    public List<ReviewSceneChangeTaskResponse> findPendingList(Long userId, ReviewSceneChangeTaskFindPendingRequest request) {
        List<ReviewSceneChangeTaskResponse> reviewSceneChangeTask = reviewSceneChangeTaskMapper.findPendingList(userId,request);

        //查询审核人员
        List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.INNOVATION_BASE_ID);
        List<SysUser> reviewerList = sysUserService.selectUserListByIds(reviewerIdList);
        Set<String> reviewerNameSet = new HashSet<>();
        for (SysUser reviewer : reviewerList){
            reviewerNameSet.add(reviewer.getNickName());
        }
        //生成创新基地人员名字字符串
        String reviewerNames = String.join("、", reviewerNameSet);

        for (ReviewSceneChangeTaskResponse taskResponse : reviewSceneChangeTask){
            taskResponse.setReviewer(reviewerNames);
        }
        return reviewSceneChangeTask;
    }

    /**
     * 创新基地查询已办列表
     * @return
     */
    @Override
    public List<ReviewSceneChangeTaskResponse> findFinishList(Long userId, ReviewSceneChangeTaskFindFinishRequest request) {
        List<ReviewSceneChangeTaskResponse> reviewSceneChangeTask = reviewSceneChangeTaskMapper.findFinishList(userId,request);

        //查询审核人员
        List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.INNOVATION_BASE_ID);
        List<SysUser> reviewerList = sysUserService.selectUserListByIds(reviewerIdList);
        Set<String> reviewerNameSet = new HashSet<>();
        for (SysUser reviewer : reviewerList){
            reviewerNameSet.add(reviewer.getNickName());
        }
        //生成审核人员名字字符串
        String reviewerNames = String.join("、", reviewerNameSet);
        for (ReviewSceneChangeTaskResponse taskResponse: reviewSceneChangeTask){
            taskResponse.setReviewer(reviewerNames);
        }
        return reviewSceneChangeTask;
    }

    @Override
    public ReviewSceneChangeTask getBySceneChange(Long id) {
        ReviewSceneChangeTask reviewSceneChangeTask = getById(id);
        //查询审核人员
        List<Long> reviewerIdList = sysUserRoleMapper.selectReviewerIdByRoleId(ReviewSceneChangeTask.INNOVATION_BASE_ID);
        List<SysUser> reviewerList = sysUserService.selectUserListByIds(reviewerIdList);

        Set<String> reviewerNameSet = new HashSet<>();
        for (SysUser reviewer : reviewerList) {
            reviewerNameSet.add(reviewer.getNickName());
        }

        // 生成审核人员名字字符串
        String reviewerNames = String.join("、", reviewerNameSet);
        reviewSceneChangeTask.setReviewer(reviewerNames);
        return reviewSceneChangeTask;
    }

    /**
     * 创新基地审核任务
     * @return
     */
    @Override
    public void innovationBaseAudit(ReviewSceneChangeTaskPassRequest request) {
        if (request.getPassed().equals(ReviewSceneChangeTask.PASSED_REJECT)) {
            innovationBaseDoProcess(request.getTaskId(), request.getComment(), ReviewSceneChangeTask.RESULT_REJECT);
        } else if(request.getPassed().equals(ReviewSceneChangeTask.PASSED_PASS)){
            innovationBaseDoProcess(request.getTaskId(), request.getComment(), ReviewSceneChangeTask.RESULT_PASS);
        } else {
            throw new ServiceException("审核状态不正确",HttpStatus.ERROR);
        }
    }
    /**
     * 创新基地评审任务
     * @param taskId
     * @param comment
     * @param decision
     */
    public void innovationBaseDoProcess(Long taskId, String comment, String decision) {

        ReviewSceneChangeTask task = getById(taskId);
        checkTaskStatus(task.getTaskStatus());

        // 获取当前登陆人
        SysUser loginUser = SecurityUtils.getLoginUser().getUser();

        // 1.获取该任务的所有审批人
        // 2.获取当前用户对应的审批信息
        // 3.保存评审意见
//        List<ReviewSceneChangeTaskUserRelation> relations = reviewSceneChangeTaskUserRelationService.findListByTaskId(taskId);
        List<ReviewSceneChangeTaskUserRelation> relations = Db.lambdaQuery(ReviewSceneChangeTaskUserRelation.class)
                .eq(ReviewSceneChangeTaskUserRelation::getTaskId, taskId)
                .eq(ReviewSceneChangeTaskUserRelation::getRoleId, ReviewSceneChangeTask.INNOVATION_BASE_ID)
                .list();
        for(ReviewSceneChangeTaskUserRelation relation : relations) {
            if(relation.getUserId().equals(loginUser.getUserId())) {
                relation.setDecision(decision);
                relation.setComment(comment);
                relation.setDecisionTime(new Date());
                reviewSceneChangeTaskUserRelationService.updateById(relation);
                break;
            }
        }

        // 判断当前任务是否全员审核完毕，如果是则给innovation_base_result赋值
        if(isFinish(relations)) {

            task.setInnovationBaseResult(isPassed(relations) ? ReviewSceneChangeTaskUserRelation.DECISION_PASS : ReviewSceneChangeTaskUserRelation.DECISION_REJECT);

            reviewSceneChangeTaskService.updateById(task);

        }

    }

    /*
     * 发起评审-新增审查场景-获取要点列表
     * */
    @Override
    public List<ReviewKeyPointListResponse> getKeyPointList(Long standardId) {
        return reviewKeyPointMapper.getKeyPointList(standardId);
    }

    /*
     * 发起评审-新增审查场景-获取细则列表
     * */
    @Override
    public List<ReviewDetailsListResponse> getDetailsList(Long keypointId) {
        return reviewKeyPointMapper.getDetailsList(keypointId);
    }
}




